<?php

namespace Andy\Calendar\Widget;

use \XF\Widget\AbstractWidget;

class Calendar extends AbstractWidget
{
    public function render()
    {
		// get options
		$options = \XF::options();
				
		// get options from Admin CP -> Options -> Calendar -> Show events maximum
		$showEventsMaximum = $options->calendarShowEventsMaximum;
		
		// get visitor
		$visitor = \XF::visitor();		
		
		// get timezone
		$timezone = $visitor['timezone'];
		
		// set timezone
		date_default_timezone_set($timezone);		
		
		// define variables
		$year = date('Y', time());	
		$month = date('m', time());
		$day = date('d', time());
		$viewParams = array();
		
		// define yearMonthDay
		$yearMonthDay = $year . $month . $day;		
				
		// get db
		$db = \XF::db();	
					
		// run query		
		$threads = $db->fetchAll("
		SELECT xf_andy_calendar.calendar_date,
		xf_node.node_id,
		xf_node.title AS nodeTitle,
		xf_thread.thread_id,
		xf_thread.title,
		xf_thread.prefix_id
		FROM xf_thread
		INNER JOIN xf_andy_calendar ON xf_andy_calendar.thread_id = xf_thread.thread_id
		INNER JOIN xf_node ON xf_node.node_id = xf_thread.node_id
		WHERE xf_thread.andy_calendar = '1'
		AND xf_andy_calendar.calendar_date >= ?
		AND xf_thread.discussion_state = 'visible'
		AND xf_thread.discussion_type <> 'redirect'			
		GROUP BY xf_thread.thread_id
		ORDER BY xf_andy_calendar.calendar_date ASC, xf_thread.title ASC
		LIMIT ?
		", array($yearMonthDay, $showEventsMaximum));
		
        // check multidimensional array
        foreach ($threads AS $k => $v)
        {
            foreach ($v as $k2 => $v2)
            {
                if ($k2 == 'node_id')
                {
					$nodePermissions = \XF::visitor()->getPermissionSet()->getContentPerms('node', $v2);	
                    
                    if ($nodePermissions['view'] == '')
                    {
						// no permission to view
                        unset($threads[$k]);
                    }
                }
				
				$count = count($threads);
				
				// don't exceed showEventsMaximum
				if ($count >= $showEventsMaximum)
				{
					break;	
				}
            }
        }
        
        // renumber array
        $threads = array_values($threads);		
				
		if (!empty($threads))
		{
			// declare variable
			$i = 0;		
			
			// add to multidimensional array
			foreach ($threads as $k => $v)
			{ 
				// prepare monthShort
				if (substr($v['calendar_date'],4,2) == '01')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_1_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '02')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_2_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '03')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_3_short'));
				}	
				
				if (substr($v['calendar_date'],4,2) == '04')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_4_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '05')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_5_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '06')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_6_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '07')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_7_short'));
				}	
				
				if (substr($v['calendar_date'],4,2) == '08')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_8_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '09')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_9_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '10')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_10_short'));
				}
				
				if (substr($v['calendar_date'],4,2) == '11')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_11_short'));
				}	
				
				if (substr($v['calendar_date'],4,2) == '12')
				{
					$monthShort = array('monthShort' => \XF::phrase('month_12_short'));
				}
				
				// prepare dayShort
				$dayShort = array('dayShort' => substr($v['calendar_date'],6,2));

				// prepare array
				$dayArray = array('dayArray' => $day);				
						
				// prepare array
				$forumArray = array('forum' => array(
					'node_id' => $v['node_id'],
					'title' => $v['nodeTitle']
				));
				
				// prepare array
				$hasPreviewArray = array('hasPreview' => true);			
	
				// merge arrays
				$calendarEvents[] = array_merge($threads[$i], $monthShort, $dayShort, $forumArray, $hasPreviewArray);				
			
				$i = $i + 1;
			}
			
			// get calendarEventsCount
			$calendarEventsCount = count($calendarEvents);

			// prepare viewParams variable
			$viewParams = [
				'calendarEvents' => $calendarEvents,
				'calendarEventsCount' => $calendarEventsCount
			];
		}
		
		return $this->renderer('andy_calendar_events', $viewParams);
    }

	public function getOptionsTemplate()
	{
	   return null;
	}
}