<?php

namespace Andy\Calendar\XF\Pub\Controller;

use XF\Mvc\ParameterBag;

class Thread extends XFCP_Thread
{
	public function actionIndex(ParameterBag $params)
	{		
		//########################################
		// show calendar dates in thread_view
		//########################################
		
		// get parent		
		$parent = parent::actionIndex($params);

        // return parent action if this is a redirect or other non View response 
        if (!$parent instanceof \XF\Mvc\Reply\View)
        {
            return $parent;
        }	
		
		// get visitor
		$visitor = \XF::visitor();		

		// check for user group permission
		if (!$visitor->hasPermission('calendar', 'view'))
		{
			return $parent;
		}
		
		// get threadId
		$threadId = $params->thread_id;
		
		//########################################
		// select calendarDates
		//########################################
		
		// get calendarDates
		$finder = \XF::finder('Andy\Calendar:Calendar');
		$calendarDates = $finder
			->where('thread_id', $threadId)
			->order('calendar_date', 'ASC')->fetch();		
		
		// declare variable
		$calendar_date = array();

		// convert multidimensional array to single arrays
		foreach ($calendarDates as $k => $v)
		{
			$calendar_date[] = $v['calendar_date'];
		}
		
		$count = count($calendar_date);
		
		//########################################
		// prepare variables
		//########################################
	
		$eventDate = '';
		
		// get options
		$options = \XF::options();		
		
		// get options from Admin CP -> Options -> Calendar -> Date Format
		$calendarDateFormat = $options->calendarDateFormat;			
		
		// get options from Admin CP -> Options -> Calendar -> Date Separator
		$calendarDateSeparator = $options->calendarDateSeparator;				
		
		// single date event
		if ($count == 1)
		{
			$fullyear = substr($calendar_date[0],0,4);
			$year = substr($calendar_date[0],2,2);
			$month = substr($calendar_date[0],4,2);
			$day = substr($calendar_date[0],6,2);
			
			if ($calendarDateFormat)
			{
				$eventDate = $day . $calendarDateSeparator . $month . $calendarDateSeparator . $year;
			}
			else
			{
				$eventDate = $month . $calendarDateSeparator . $day . $calendarDateSeparator . $year;
			}
		}

		// multiple date events
		if ($count > 1)
		{
			for ($i=0; $i<$count; $i++)
			{
				$fullyear = substr($calendar_date[$i],0,4);
				$year = substr($calendar_date[$i],2,2);
				$month = substr($calendar_date[$i],4,2);
				$day = substr($calendar_date[$i],6,2);
				
				if ($calendarDateFormat)
				{
					$date = $day . $calendarDateSeparator . $month . $calendarDateSeparator . $year;
				}
				else
				{
					$date = $month . $calendarDateSeparator . $day . $calendarDateSeparator . $year;
				}				
				
				// create eventDate variable
				$eventDate = $eventDate . $date;
				
				// add comma if not last event date
				if ($i < ($count - 1))
				{
					$eventDate = $eventDate . ', ';
				}
			}
		}

		// setParams
		$parent->setParams([
			'eventDate' => $eventDate
		]);
		
		// return parent
		return $parent;
	}
	
	public function actionCalendarAdd(ParameterBag $params)
	{
		//########################################
		// calendar add
		//########################################	
					
		// get visitor
		$visitor = \XF::visitor();
		
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			return $this->actionRegistrationRequired();	
		}	
		
		// get thread
		$thread = $this->assertViewableThread($params->thread_id);			

		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread['user_id'])
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}

		// prepare viewParams variable
		$viewParams = [
			'thread' => $thread	
		];
		
		// send to template
		return $this->view('Andy\Calendar:AndyCalendarAdd', 'andy_calendar_add', $viewParams);		
	}
	
	public function actionCalendarAddSave(ParameterBag $params)
	{
		//########################################
		// calendar add save
		//########################################	
		
		// get visitor
		$visitor = \XF::visitor();		
			
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			throw $this->getNoPermissionResponseException();
		}
		
		$this->assertPostOnly();
		
		// get thread
		$thread = $this->assertViewableThread($params->thread_id);
		
		// get threadId
		$threadId = $thread['thread_id'];	
		
		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread->user_id)
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}
		
		// assert post only
		$this->assertPostOnly();	
		
		// get eventDate
		$eventDate = $this->filter('event_date', 'str');

		// return error if no eventDate
		if ($eventDate == '')
		{
			return $this->error(\XF::phrase('calendar_event_date_not_entered'));
		}
		
		// get eventEndDate
		$eventEndDate = $this->filter('event_end_date', 'str');
		
		//########################################
		// get eventDays
		//########################################
		
		// convert to unix timestamp
		$eventDateTime = strtotime($eventDate);
		$eventEndDateTime = strtotime($eventEndDate);
		
		// set default eventDays
		$eventDays = 1;

		if ($eventEndDate != '')
		{
			// get var
			$var = $eventEndDateTime - $eventDateTime;

			// get eventDays 
			$eventDays = floor($var / (60 * 60 * 24));

			// add 1 to eventDays
			$eventDays = $eventDays + 1;

			// check for event end date error
			if ($eventDays <= 0)
			{
				return $this->error(\XF::phrase('calendar_event_end_date_error'));
			}

			// check for too many days selected
			if ($eventDays >= 60)
			{
				return $this->error(\XF::phrase('calendar_too_many_days_selected_error'));
			}
		}
		
		// continue if threadId is numeric
		if (is_numeric($threadId))
		{
			//########################################
			// update
			//########################################
			
			// prepare data
			$data = [
				'andy_calendar' => 1
			];

			// update xf_thread
			$thread->fastUpdate($data);
			
			// add event dates
			for ($i=0; $i<$eventDays; $i++)
			{
				// each loop will update to next day
				$eventDateTimeNew = $eventDateTime + (60 * 60 * 24 * $i);

				// convert to date format
				$date = date('Ymd', $eventDateTimeNew);
				
				//########################################
				// insert
				//########################################				

				// create row in xf_andy_calendar
				$calendar = $this->em()->create('Andy\Calendar:Calendar');
				$calendar->thread_id = $threadId;
				$calendar->calendar_date = $date;
				$calendar->user_id = $userId;
				$calendar->save();
			}

			// return redirect
			return $this->redirect(
				$this->getDynamicRedirect($this->buildLink('threads', $thread), true)
			);
		}
	}
	
	public function actionCalendarAddRecurring(ParameterBag $params)
	{
		//########################################
		// calendar add recurring
		//########################################	
					
		// get visitor
		$visitor = \XF::visitor();
		
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			return $this->actionRegistrationRequired();	
		}	
		
		// get thread
		$thread = $this->assertViewableThread($params->thread_id);			

		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread['user_id'])
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}

		// prepare viewParams variable
		$viewParams = [
			'thread' => $thread	
		];
		
		// send to template
		return $this->view('Andy\Calendar:AndyCalendarAdd', 'andy_calendar_add_recurring', $viewParams);		
	}
	
	public function actionCalendarAddSaveRecurring(ParameterBag $params)
	{
		//########################################
		// calendar add save recurring
		//########################################	
		
		// get visitor
		$visitor = \XF::visitor();		
			
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			throw $this->getNoPermissionResponseException();
		}
		
		$this->assertPostOnly();
		
		// get thread
		$thread = $this->assertViewableThread($params->thread_id);
		
		// get threadId
		$threadId = $thread['thread_id'];	
		
		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread->user_id)
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}
		
		// assert post only
		$this->assertPostOnly();	
		
		// get eventDate
		$eventStartDate = $this->filter('event_start_date', 'str');

		// return error if no eventStartDate
		if ($eventStartDate == '')
		{
			return $this->error(\XF::phrase('calendar_event_date_not_entered'));
		}
		
		// get eventEndDate
		$eventEndDate = $this->filter('event_end_date', 'str');
		
		//########################################
		// get eventDays
		//########################################
		
		// convert to unix timestamp
		$eventStartDateTime = strtotime($eventStartDate);
		$eventEndDateTime = strtotime($eventEndDate);
		
		// set default eventDays
		$eventDays = 1;

		if ($eventEndDate != '')
		{
			// get var
			$var = $eventEndDateTime - $eventStartDateTime;

			// get eventDays 
			$eventDays = floor($var / (60 * 60 * 24 * 7));

			// add 1 to eventDays
			$eventDays = $eventDays + 1;

			// check for event end date error
			if ($eventDays <= 0)
			{
				return $this->error(\XF::phrase('calendar_event_end_date_error'));
			}

			// check for too many days selected
			if ($eventDays >= 60)
			{
				return $this->error(\XF::phrase('calendar_too_many_days_selected_error'));
			}
		}
		
		// continue if threadId is numeric
		if (is_numeric($threadId))
		{
			//########################################
			// update
			//########################################
			
			// prepare data
			$data = [
				'andy_calendar' => 1
			];

			// update xf_thread
			$thread->fastUpdate($data);
			
			// add event dates
			for ($i=0; $i<$eventDays; $i++)
			{
				// each loop will update to next day
				$eventDateTimeNew = $eventStartDateTime + (60 * 60 * 24 * 7 * $i);

				// convert to date format
				$date = date('Ymd', $eventDateTimeNew);
				
				//########################################
				// insert
				//########################################				

				// create row in xf_andy_calendar
				$calendar = $this->em()->create('Andy\Calendar:Calendar');
				$calendar->thread_id = $threadId;
				$calendar->calendar_date = $date;
				$calendar->user_id = $userId;
				$calendar->save();
			}

			// return redirect
			return $this->redirect(
				$this->getDynamicRedirect($this->buildLink('threads', $thread), true)
			);
		}
	}
	
	public function actionCalendarRemove(ParameterBag $params)
	{
		//########################################
		// calendar remove
		//########################################
		
		// get visitor
		$visitor = \XF::visitor();
		
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			return $this->actionRegistrationRequired();
		}		
		
		// get thread
		$thread = $this->assertViewableThread($params->thread_id);
		
		// get threadId
		$threadId = $thread['thread_id'];

		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread->user_id)
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}

		// get db
		$db = \XF::db();
		
        // run query
        $calendarDates = $db->fetchAll("
        SELECT thread_id,
		calendar_date
        FROM xf_andy_calendar
		WHERE thread_id = ?
        ORDER BY calendar_date ASC
        ", $threadId);

		//########################################
		// prepare variables
		//########################################
	
		$eventDate = '';
		
		// get options
		$options = \XF::options();		
		
		// get options from Admin CP -> Options -> Calendar -> Date Format
		$calendarDateFormat = $options->calendarDateFormat;			
		
		// get options from Admin CP -> Options -> Calendar -> Date Separator
		$calendarDateSeparator = $options->calendarDateSeparator;				
		
		// add to multidimensional array
		foreach ($calendarDates as $k => $v)
		{
			$fullyear = substr($v['calendar_date'],0,4);
			$year = substr($v['calendar_date'],2,2);
			$month = substr($v['calendar_date'],4,2);
			$day = substr($v['calendar_date'],6,2);
				
			if ($calendarDateFormat)
			{
				$calendarDates[$k]['calendar_date_formatted'] = $day . $calendarDateSeparator . $month . $calendarDateSeparator . $year;
			}
			else
			{
				$calendarDates[$k]['calendar_date_formatted'] = $month . $calendarDateSeparator . $day . $calendarDateSeparator . $year;
			}
		}
		
		// rename variable
		$results = $calendarDates;
		
		// define variable
		$multiple_dates = false;
		
		// set multiple_dates
		if (count($results) > 1)
		{
			$multiple_dates = true;
		}
		
		// prepare viewParams
		$viewParams = [
			'thread' => $thread,
			'results' => $results,
			'multiple_dates' => $multiple_dates
		];
		
		// send to template
		return $this->view('Andy\Calendar:AndyCalendarRemove', 'andy_calendar_remove', $viewParams);			
	}
	
	public function actionCalendarRemoveAll(ParameterBag $params)
	{
		//########################################
		// calendar remove save
		//########################################
				
		// get visitor
		$visitor = \XF::visitor();
		
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			return $this->actionRegistrationRequired();
		}
		
		// get threadId
		$threadId = $this->filter('thread_id', 'uint');
		
		// get thread
		$thread = $this->assertViewableThread($threadId);

		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread->user_id)
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}	
		
		// remove from calendar if threadId is numeric
		if (is_numeric($thread['thread_id']))
		{
			//########################################
			// update
			//########################################
			
			// prepare data
			$data = [
				'andy_calendar' => 0
			];

			// update xf_thread
			$thread->fastUpdate($data);
			
			//########################################
			// delete
			//########################################
			
			$finder = \XF::finder('Andy\Calendar:Calendar');
			$threadIds = $finder
				->where('thread_id', $threadId)
				->fetch();
			
			foreach ($threadIds AS $thread)
			{
				$thread->delete();
			}			
			
			//########################################
			// redirect
			//########################################
			
			return $this->redirect(
				$this->getDynamicRedirect($this->buildLink('threads', $thread), false)
			);			
		}
	}
	
	public function actionCalendarRemoveSingle(ParameterBag $params)
	{
		//########################################
		// calendar remove single
		//########################################
				
		// get visitor
		$visitor = \XF::visitor();
		
		// get userId
		$userId = $visitor['user_id'];
		
		// throw error if no userId
		if ($userId == '')
		{
			return $this->actionRegistrationRequired();
		}
		
		// get threadId
		$threadId = $this->filter('thread_id', 'uint');
		
		// get calendarDate
		$calendarDate = $this->filter('calendar_date', 'uint');
		
		// get thread
		$thread = $this->assertViewableThread($threadId);

		// declare variable
		$authorized = '';			
		
		// get permission
		$addRemoveAll = $visitor->hasPermission('calendar', 'addRemoveAll');
		
		// get permission
		$addRemoveOwn = $visitor->hasPermission('calendar', 'addRemoveOwn');		
		
		// show authorized if addRemoveAll
		if ($addRemoveAll)
		{
			$authorized = 'yes';
		}			
		
		// show authorized if addRemoveOwn and thread starter
		if ($addRemoveOwn AND $userId == $thread->user_id)
		{
			$authorized = 'yes';
		}			
		
		// throw error if not authorized
		if ($authorized == '')
		{
			return $this->noPermission();
		}	
		
		// remove from calendar if threadId is numeric
		if (is_numeric($threadId))
		{	
			//########################################
			// delete
			//########################################
			
			// get db
			$db = \XF::db();

			// delete row
			$db->query("
				DELETE FROM xf_andy_calendar
				WHERE thread_id = ?
				AND calendar_date = ?
			", array($threadId, $calendarDate));		
			
			//########################################
			// redirect
			//########################################
			
			return $this->redirect(
				$this->getDynamicRedirect($this->buildLink('threads', $thread), false)
			);			
		}
	}
}